#! /usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2011 ~ 2014 Deepin, Inc.
#               2013 ~ 2014 Kaisheng Ye
#
# Author:     Kaisheng Ye <kaisheng.ye@gmail.com>
# Maintainer: Kaisheng Ye <kaisheng.ye@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
import urllib2
import urllib
import os
import commands
import tarfile

from deepin_utils.hash import md5_file
from deepin_utils.file import create_directory, get_parent_dir
from deepin_utils.date_time import get_current_time

BUG_HOST = "http://package-screenshot.b0.upaiyun.com"

system_modules_dir = os.path.join(get_parent_dir(__file__, 2), "modules")
local_modules_dir = os.path.join(os.path.expanduser("~"), ".config/deepin-bug-reporter/modules")

def run(module_name=None):
    modules = ['systeminfo']
    if module_name:
        modules.append(module_name)

    print "update module scripts..."
    newest_module_paths = []
    for name in modules:
        module_path = update_module(name)
        if module_path != "":
            newest_module_paths.append(module_path)
    print "finish module scripts"

    print "generate logs for every module..."
    log_paths = []
    for script_path in newest_module_paths:
        out_path = commands.getoutput("python %s" % script_path)
        log_paths.append(out_path)
    print "finish generating logs for every module..."

    print "compress log files..."
    currnet_time_str = get_current_time("%Y%m%d%H%M%S")
    if module_name:
        tar_name = "deepin_logs_%s_%s.tar.gz" % (module_name, currnet_time_str)
    else:
        tar_name = "deepin_logs_%s.tar.gz" % currnet_time_str
    tar_dir = os.path.join(os.path.expanduser("~"), "deepin-bug-reporter")
    tar_path = os.path.join(tar_dir, tar_name)
    create_directory(tar_dir)
    with tarfile.open(tar_path, "w:gz") as tar:
        for fullpath in log_paths:
            tar.add(fullpath, os.path.split(fullpath)[1] , False)
    print "Finish compressing at:"
    print tar_path

def update_module(name):
    md5_url = "%s/modules/%s/md5.txt" % (BUG_HOST, name)
    module_script_url = "%s/modules/%s/reporter.py" % (BUG_HOST, name)
    local_module_script = os.path.join(local_modules_dir, "%s.py" % name)
    system_module_script = os.path.join(system_modules_dir, "%s.py" % name)

    old_path = ""
    for path in [local_module_script, system_module_script]:
        if os.path.exists(path):
            old_path = path

    try:
        remote_md5 = urllib2.urlopen(md5_url).read()
        need_download = True

        if old_path != "":
            local_md5 = md5_file(old_path)
            if local_md5 == remote_md5:
                need_download = False

        if need_download:
            try:
                create_directory(local_modules_dir)
                urllib.urlretrieve(module_script_url, local_module_script)
                return local_module_script
            except Exception:
                pass
    except Exception:
        pass
    return old_path


if __name__ == "__main__":
    if len(sys.argv) == 2:
        run(sys.argv[1])
    else:
        run()
